<?php

namespace Drupal\opensearchtab\Controller;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Controller\ControllerBase;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\Response;
use Symfony\Component\HttpFoundation\Request;

/**
 * OpenSearchDescriptionController .
 */
class OpenSearchDescriptionController extends ControllerBase {

  /**
   * System configurations.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $systemConfig;

  /**
   * Open Search Tab configurations.
   *
   * @var \Drupal\Core\Config\ImmutableConfig
   */
  protected $ostConfig;

  /**
   * A request object.
   *
   * @var \Symfony\Component\HttpFoundation\Request
   */
  protected $request;

  /**
   * A response object.
   *
   * @var \Symfony\Component\HttpFoundation\Response
   */
  protected $response;

  /**
   * Class constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory service.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->systemConfig = $config_factory->get('system.site');
    $this->ostConfig = $config_factory->get('opensearchtab.settings');
    $this->request = new Request();
    $this->response = new Response();
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function content() {
    // Prepare variables for the XML content.
    $short_name = !empty($this->ostConfig->get('shortname')) ?
      $this->ostConfig->get('shortname') :
      (!empty($this->systemConfig->get('name')) ?
        $this->systemConfig->get('name') : 'Drupal site');
    $description = !empty($this->ostConfig->get('description')) ?
      $this->ostConfig->get('description') :
      (!empty($this->systemConfig->get('slogan')) ?
        $this->systemConfig->get('slogan') : 'Search the site');
    $search_form_path = !empty($this->ostConfig->get('search-form-path')) ?
      $this->request->getSchemeAndHttpHost() .
      $this->ostConfig->get('search-form-path') :
      $this->request->getSchemeAndHttpHost() . '/search/node';
    $search_path = !empty($this->ostConfig->get('search-path')) ?
      $this->request->getSchemeAndHttpHost() .
      $this->ostConfig->get('search-path') :
      $this->request->getSchemeAndHttpHost() .
      '/search/node?keys={searchTerms}';
    $suggestions_path_json = !empty($this->ostConfig->get('suggestions-path-json')) ?
      $this->request->getSchemeAndHttpHost() .
      $this->ostConfig->get('suggestions-path-json') : '';
    $suggestions_path_xml = !empty($this->ostConfig->get('suggestions-path-xml')) ?
      $this->request->getSchemeAndHttpHost() .
      $this->ostConfig->get('suggestions-path-xml') : '';
    $favicon_path = !empty($this->ostConfig->get('favicon-path')) ?
      $this->request->getSchemeAndHttpHost() .
      $this->ostConfig->get('favicon-path') : '';

    // Define the XML content.
    $openTag = new \SimpleXMLElement(
      '<OpenSearchDescription></OpenSearchDescription>'
    );
    $openTag->addAttribute(
      'xmlns', 'http://a9.com/-/spec/opensearch/1.1/'
    );
    $openTag->addAttribute(
      'xmlns:xmlns:moz',
      'http://www.mozilla.org/2006/browser/search/'
    );
    $openTag->addChild('ShortName', $short_name);
    $openTag->addChild('Description', $description);
    $searchUrl = $openTag->addChild('Url');
    $searchUrl->addAttribute('type', 'text/html');
    $searchUrl->addAttribute('method', 'get');
    $searchUrl->addAttribute('template', $search_path);
    if ($suggestions_path_json !== '') {
      $suggestionsUrl = $openTag->addChild('Url');
      $suggestionsUrl->addAttribute(
        'type', 'application/x-suggestions+json'
      );
      $suggestionsUrl->addAttribute('method', 'get');
      $suggestionsUrl->addAttribute('template', $suggestions_path_json);
    }
    if ($suggestions_path_xml !== '') {
      $suggestionsUrl = $openTag->addChild('Url');
      $suggestionsUrl->addAttribute(
        'type', 'application/x-suggestions+xml'
      );
      $suggestionsUrl->addAttribute('method', 'get');
      $suggestionsUrl->addAttribute('template', $suggestions_path_xml);
    }
    if ($favicon_path !== '') {
      $faviconUrl = $openTag->addChild('Image', $favicon_path);
      $faviconUrl->addAttribute('height', '16');
      $faviconUrl->addAttribute('width', '16');
      $faviconUrl->addAttribute('type', 'image/x-icon');
    }
    $openTag->addChild('xmlns:moz:SearchForm', $search_form_path);

    // Prepare the response.
    $this->response->setContent($openTag->asXML());
    $this->response->headers->set('Content-type', 'text/xml');

    return $this->response;
  }

}
